__title__ = "logiclayer-merge"
__description__ = "Allows to merge the results of other LogicLayer modules in a single response."
__version__ = "0.1.0"

__all__ = ("MergeModule", "ResponseFormat")

from .module import MergeModule
